﻿local L = LibStub("AceLocale-3.0"):GetLocale("Chinon")

local maxX=0;
local maxY=0;
local nombreElements={};

local boutonMarge = 6;
local boutonMargeHaut = 7
local boutonEspace = 3;

local nombreLignes

--[[Ajoute un nouveau bouton
\param y La ligne à laquelle ajouter le bouton
\param cfgBouton La configuration du bouton]]
function SoinFrame:AjouterBouton(cible, y, cfgBouton)
	local nom = "bouton"..cible;
	
	nom = nom .. "_" .. cfgBouton.nom;
	
	local h=self.boutons[nom];
	if (not h) then
		h=CreateFrame("Button",nil,self,"SoinBouton");
		SoinBouton_Init(h)
		self.boutons[nom]=h;
	end
	
	for i, sort in pairs(cfgBouton.clic) do
		SoinBouton_AddSort(h, sort, cible);
	end
	
	h:Configure(cfgBouton);
	h:SetAttribute("unit",cible);

	
	local x = nombreElements[y]
	if (not x) then
		x=0
	end
	nombreElements[y] = x+1
		
	local left = x*(Chinon.apparence.largeurBouton+boutonEspace)+boutonMarge;
	local top = y*(Chinon.apparence.hauteurBouton+boutonEspace)+Chinon.apparence.hauteurBoutonCD+boutonMargeHaut*2;
	if (left>maxX) then
		maxX=left;
	end
	if (top>maxY) then
		maxY=top;
	end
	h:SetPoint("TOPLEFT", self, "TOPLEFT", left, -top);
	h:Show();
end

--[[A appeler pour initialiser la frame ]]
function SoinFrame:Load()
	if (Chinon.cfgBoutons == nil) then
		self:Hide();
		return;
	end

   
	self.degats = {}
	self.totalDegats = {}
	self.boutons = {}
	self.boutonsCD = {}
	self.boutonsSB = {}
	
	for sort,niveau in pairs(chinonCouleurDebuff) do
		local nom = GetSpellInfo(sort);
		Chinon.decurseLevel[nom] = niveau;
	end

	self.labelLigne={};
	
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("PARTY_MEMBER_ENABLE");
	self:RegisterEvent("PARTY_MEMBER_DISABLE");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("UNIT_COMBAT");
	self:RegisterEvent("UNIT_PET");

	self:Update(self);
end

function SoinFrame:OnMouseDown(button)
	if ( not self.enCombat and button == "LeftButton") then
		self:StartMoving();
	end
end

function SoinFrame:OnMouseUp(button)
	if ( button == "LeftButton" ) then	
		self:StopMovingOrSizing();
	end
end

function SoinFrame:SetLabel(i, text)
	if (not self.labelLigne[i]) then
		self.labelLigne[i]=self:CreateFontString(nil);
		self.labelLigne[i]:SetFontObject("GameFontNormal");
		self.labelLigne[i]:SetTextColor(1,1,1);
	end
	self.labelLigne[i]:SetPoint("TOPRIGHT",self,"TOPLEFT",0,
		(i)*-(boutonEspace+Chinon.apparence.hauteurBouton) - 
		(Chinon.apparence.hauteurBoutonCD+boutonMargeHaut*2));
	self.labelLigne[i]:Show()
	self.labelLigne[i]:SetText(text)
end

function SoinFrame:AddMemberLine(unit, groupe, arene)
	if (UnitExists(unit)) then
		self:SetLabel(nombreLignes, UnitName(unit));
		for j, cfgBouton in ipairs(Chinon.cfgBoutons) do
			if (groupe and cfgBouton.groupe) then
				self:AjouterBouton(unit,nombreLignes,cfgBouton);
			end
			if (arene and cfgBouton.arene) then
				self:AjouterBouton(unit,nombreLignes,cfgBouton);
			end
		end
		nombreLignes = nombreLignes +1
	end
end

function SoinFrame:AddPartyMemberLine(unit)
	self:AddMemberLine(unit, true, false)
end

function SoinFrame:AddArenaMemberLine(unit)
	self:AddMemberLine(unit, false, true)
end

-- Recrée les boutons dans le cas d'un groupe
function SoinFrame:UpdateParty()
	if (Chinon.apparence.hideLigneCible or #Chinon.cfgBoutons>1) then
		self:AddPartyMemberLine("player")
	end
	for i=1,GetNumPartyMembers() do
		self:AddPartyMemberLine("party"..i)
	end
	self:AddPartyMemberLine("pet")
	for i=1,GetNumPartyMembers() do
		self:AddPartyMemberLine("partypet"..i)
	end
end

-- Recrée les boutons dans le cas d'un raid de
-- 5 joueurs ou moins (arène)
function SoinFrame:UpdateArena()
	for i=1,GetNumRaidMembers() do
		self:AddArenaMemberLine("raid"..i)
	end
	for i=1,GetNumRaidMembers() do
		self:AddArenaMemberLine("raidpet"..i)
	end
end

-- Recrée les boutons en raid
function SoinFrame:UpdateRaid()
	local sortClasse;
	if (Chinon.apparence.classerParGroupe) then
		sortClasse=false;
	else
		sortClasse=true;
	end
	
	local ligneClasse = {};
	
	local baseLigne = nombreLignes

	for i=1,GetNumRaidMembers() do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i);
		local ligne

		if (subgroup and fileName) then
			if (sortClasse) then
				if (not ligneClasse[fileName]) then
					ligneClasse[fileName] = nombreLignes
					self:SetLabel(nombreLignes,class);
					nombreLignes = nombreLignes + 1
				end
				ligne = ligneClasse[fileName];
			else
				ligne = subgroup-1+baseLigne;
				self:SetLabel(ligne,subgroup)
				if (ligne>=nombreLignes) then
					nombreLignes = ligne+1
				end
			end
			for k, cfgBouton in ipairs(Chinon.cfgBoutons) do
				if (cfgBouton.raid) then
					self:AjouterBouton("raid"..i, ligne, cfgBouton);
				end
			end
		end
	end

	local familiers = 0	
	for i=1,GetNumRaidMembers() do
		if (UnitExists("raidpet"..i)) then
			if (familiers == 5) then
				familiers = 0
				nombreLignes = nombreLignes + 1
			end
			
			if (familiers == 0) then
				self:SetLabel(nombreLignes,L["Familier"]);
			end
			
			for k, cfgBouton in ipairs(Chinon.cfgBoutons) do
				if (cfgBouton.raid) then
					self:AjouterBouton("raidpet"..i, nombreLignes, cfgBouton);
				end
			end
			familiers = familiers + 1
		end
	end
	
	if (familiers>0) then
		nombreLignes = nombreLignes + 1
	end
end

function SoinFrame:AjouterBoutonCD(i, sort)
	if (not self.boutonsCD[i]) then
		self.boutonsCD[i] = CreateFrame("Frame",nil,self,nil)
		self.boutonsCD[i].icon = self.boutonsCD[i]:CreateTexture();
		self.boutonsCD[i].icon:SetDrawLayer("ARTWORK");
		self.boutonsCD[i].icon:SetAllPoints(self.boutonsCD[i]);
		self.boutonsCD[i].cd = CreateFrame("Cooldown",nil,self.boutonsCD[i],nil);
		self.boutonsCD[i].cd:SetAllPoints(self.boutonsCD[i]);
	end
	self.boutonsCD[i].sort = Chinon:GetSpellInfoOrNil(sort)
	self.boutonsCD[i].icon:SetTexture(GetSpellTexture(self.boutonsCD[i].sort, BOOKTYPE_SPELL))
	self.boutonsCD[i]:SetPoint("TOPLEFT",(i-1)*Chinon.apparence.largeurBoutonCD+boutonMargeHaut,-boutonMargeHaut)
	self.boutonsCD[i]:SetHeight(Chinon.apparence.hauteurBoutonCD)
	self.boutonsCD[i]:SetWidth(Chinon.apparence.largeurBoutonCD)
	self.boutonsCD[i]:Show()
	self.boutonsCD[i].cd:Show()
	self.boutonsCD[i].icon:Show()
end

function SoinFrame:AjouterBoutonSB(i, sort)
	if (not self.boutonsSB[i]) then
		self.boutonsSB[i] = CreateFrame("Frame",nil,self,nil)
		self.boutonsSB[i].icon = self.boutonsSB[i]:CreateTexture();
		self.boutonsSB[i].icon:SetDrawLayer("ARTWORK");
		self.boutonsSB[i].icon:SetAllPoints(self.boutonsSB[i]);
	end
	local nomSort = Chinon:GetSpellInfoOrNil(sort)
	if (nomSort) then
		self.sbBouton[nomSort]=i
		self.boutonsSB[i].texture = GetSpellTexture(nomSort, BOOKTYPE_SPELL)
		if (self.boutonsSB[i].texture) then
			self.boutonsSB[i].icon:SetTexture(self.boutonsSB[i].texture)
		end
		self.boutonsSB[i]:SetPoint("TOPLEFT",(i+#Chinon.cfgBoutonsCD-0.5)*Chinon.apparence.largeurBoutonCD+boutonMargeHaut,-boutonMargeHaut)
		self.boutonsSB[i]:SetHeight(Chinon.apparence.hauteurBoutonCD)
		self.boutonsSB[i]:SetWidth(Chinon.apparence.largeurBoutonCD)
		self.boutonsSB[i]:Show()
		self.boutonsSB[i].icon:Show()
	end
end

-- Recrée les boutons
function SoinFrame:Update()
	local i=1;
	
	nombreLignes = 0
	
	if (not self.boutons) then
		self:Hide()
		return;
	end


	Chinon:AppliquerFrameSkin(self, Chinon.apparence.frameSkin)

	for i, bouton in pairs(Chinon.cfgBoutons) do
		Chinon:UpdateDebuffsBouton(bouton);
		Chinon:UpdateBuffsBouton(bouton) 
		Chinon:UpdateDebuffs2Bouton(bouton)
	end
	
	for i, boutonCD in pairs(self.boutonsCD) do
		boutonCD.icon:Hide()
		boutonCD.cd:Hide()
		boutonCD:Hide()
	end

	for i, boutonSB in pairs(self.boutonsSB) do
		boutonSB.icon:Hide()
		boutonSB:Hide()
	end
	
	for i, sort in ipairs(Chinon.cfgBoutonsCD) do
		self:AjouterBoutonCD(i, sort)
	end
	
	self.sbBouton = {}
	
	for i, sort in ipairs(Chinon.cfgBoutonsSB) do
		self:AjouterBoutonSB(i, sort)
	end
	
	for i, bouton in pairs(self.boutons) do
		bouton:Hide();
	end
	
	for i, label in pairs(self.labelLigne) do
		if (self.labelLigne[i]) then
			label:SetPoint("TOPRIGHT",self,"TOPLEFT",0,i*-(boutonEspace+Chinon.apparence.hauteurBouton) -18);
			label:Hide()
		end
	end
	
	nombreElements={}
	
	maxX = 3*(Chinon.apparence.largeurBouton+boutonEspace)+boutonMarge;
	maxY = Chinon.apparence.hauteurBoutonCD+boutonMargeHaut*2;
	
	-- Quatre boutons de base
	if (Chinon.cfgBoutons[1] and not Chinon.apparence.hideLigneCible) then
		self:AjouterBouton("player",nombreLignes,Chinon.cfgBoutons[1]);
		self:AjouterBouton("target",nombreLignes,Chinon.cfgBoutons[1]);
		self:AjouterBouton("targettarget",nombreLignes,Chinon.cfgBoutons[1]);
		self:AjouterBouton("focus",nombreLignes,Chinon.cfgBoutons[1]);
		nombreLignes = nombreLignes + 1
	end

	if (GetNumRaidMembers()>5) then
		self:UpdateRaid();
	elseif (GetNumRaidMembers()>0) then
		self:UpdateArena();
	else
		self:UpdateParty();
	end
	
	if (Chinon.apparence.showGroupeSeulement and GetNumRaidMembers()==0 and GetNumPartyMembers()==0) then
		self:Hide()
	else
		self:Show()
	end
	
	-- On redimentionne le fond de la fenêtre pour
	-- englober tous les boutons
	self:SetWidth(maxX+boutonMarge+Chinon.apparence.largeurBouton);
	self:SetHeight(maxY+boutonMarge+Chinon.apparence.hauteurBouton);
	
	ChinonDebuffs:UpdateSkin()
end

function SoinFrame:OnEvent(event)
	if (event == "PLAYER_REGEN_DISABLED") then
		self.enCombat=true;
		self:StopMovingOrSizing();
	end
	if (event == "PLAYER_REGEN_ENABLED") then
		self.enCombat=false;
		self:Update();
	end 
	if (event == "PARTY_MEMBERS_CHANGED" or event == "PARTY_MEMBER_ENABLE"
		or event == "PARTY_MEMBER_DISABLE" or event == "RAID_ROSTER_UPDATE"
		or event=="UNIT_PET") then
		if (not self.enCombat) then
			self:Update();
		end	
	end

	if (event == "UNIT_COMBAT") then
		if (UnitIsFriend("player",arg1)) then
			if (not self.degats[arg1]) then
				self.degats[arg1]={};
			end
			local time=GetTime();
			local valeur;
			if (arg2 == "HEAL") then
				valeur=0
			else
				valeur=arg4;
			end
			
			if (not self.degats[arg1][time]) then
				self.degats[arg1][time]=valeur;
			else
				self.degats[arg1][time]=self.degats[arg1][time]+valeur;
			end
		end
	end
end

function SoinFrame:SetSoin(cible, combien)
	for i, bouton in pairs(self.boutons) do
		local target = bouton:GetAttribute("unit")
		local name, realm = UnitName(target)
		if (realm) then
			name = name..'-'..realm
		end
		if (name == cible) then
			bouton.soin = bouton.soin + combien
		end
	end
	
end

function SoinFrame:OnUpdate()
	local t=GetTime();
	
	if (not Chinon.cfgBoutons) then
		return;
	end
	
	-- local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo("target");
	-- if (attention[spell]) then
	--	SoinFrame_Info:SetText(spell);
	-- else
	--	SoinFrame_Info:SetText("");
	-- end
	
	for cible,tab in pairs(self.degats) do
		local perdu=0;
		for temps,valeur in pairs(tab) do
			if (temps<t-4) then
				tab[temps]=nil;
			else
				perdu=perdu+valeur;
			end
		end
		self.totalDegats[cible] = perdu
	end
	
	for i, sort in ipairs(Chinon.cfgBoutonsCD) do
		local start, duration, enabled = GetSpellCooldown(self.boutonsCD[i].sort, BOOKTYPE_SPELL)
		if (start) then
			self.boutonsCD[i].cd:SetCooldown(start, duration)
		end
	end
	
	if (#Chinon.cfgBoutonsSB>0) then
		for key, value in pairs(self.boutonsSB) do
			value.wasSet = value.set
			value.set = false
		end
		
		local i=1
		while 1 do
			local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable =  UnitBuff("player", i);
			if (not name) then
				break;
			end
			
			if (self.sbBouton[name]) then
				--local start = expirationTime - duration
				--self.boutonsSB[self.sbBouton[name]].cd:SetCooldown(start, duration)
				local bouton = self.boutonsSB[self.sbBouton[name]]
				if (not bouton.texture) then
					bouton.texture = icon
					bouton.icon:SetTexture(icon)
				end
				local remains = expirationTime - t
				if (remains<3 and duration>0) then
					remains = remains * 4
					remains = remains - math.floor(remains)
					if (remains<0.5) then
						bouton.set =  true
					else
						bouton.set = false
					end
				else
					bouton.set = true
				end
			end
			i=i+1
		end
		
		for key, value in pairs(self.boutonsSB) do
			if (value.wasSet ~= value.set) then
				if (not value.set) then
					value.icon:SetAlpha(0.25)
					--value.cd:Hide()
				else
					value.icon:SetAlpha(1)
					--value.cd:Show()
				end
			end
		end
	end
end

